@@ -23,7 +23,8 @@ import (
 	"strconv"
 	"strings"
 
-	"k8s.io/api/core/v1"
+	v1 "k8s.io/api/core/v1"
+	"k8s.io/apimachinery/pkg/util/sets"
 	runtimeapi "k8s.io/cri-api/pkg/apis/runtime/v1"
 )
 
@@ -39,7 +40,7 @@ func (i *internalContainerLifecycleImpl) PreCreateContainer(pod *v1.Pod, contain
 		numaNodes := i.memoryManager.GetMemoryNUMANodes(pod, container)
 		if numaNodes.Len() > 0 {
 			var affinity []string
-			for _, numaNode := range numaNodes.List() {
+			for _, numaNode := range sets.List(numaNodes) {
 				affinity = append(affinity, strconv.Itoa(numaNode))
 			}
 			containerConfig.Linux.Resources.CpusetMems = strings.Join(affinity, ",")
