@@ -18,85 +18,76 @@ package nodeaffinity
 
 import (
 	"context"
-	"reflect"
+	"fmt"
 	"testing"
 
 	"github.com/google/go-cmp/cmp"
+	"github.com/stretchr/testify/require"
 	v1 "k8s.io/api/core/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
+	"k8s.io/apimachinery/pkg/util/sets"
+	"k8s.io/klog/v2/ktesting"
 	"k8s.io/kubernetes/pkg/scheduler/apis/config"
 	"k8s.io/kubernetes/pkg/scheduler/framework"
 	"k8s.io/kubernetes/pkg/scheduler/framework/runtime"
 	"k8s.io/kubernetes/pkg/scheduler/internal/cache"
+	st "k8s.io/kubernetes/pkg/scheduler/testing"
+	tf "k8s.io/kubernetes/pkg/scheduler/testing/framework"
 )
 
 // TODO: Add test case for RequiredDuringSchedulingRequiredDuringExecution after it's implemented.
 func TestNodeAffinity(t *testing.T) {
 	tests := []struct {
-		name             string
-		pod              *v1.Pod
-		labels           map[string]string
-		nodeName         string
-		wantStatus       *framework.Status
-		args             config.NodeAffinityArgs
-		disablePreFilter bool
+		name                string
+		pod                 *v1.Pod
+		labels              map[string]string
+		nodeName            string
+		wantStatus          *framework.Status
+		wantPreFilterStatus *framework.Status
+		wantPreFilterResult *framework.PreFilterResult
+		args                config.NodeAffinityArgs
+		runPreFilter        bool
 	}{
-		{
-			name: "no selector",
-			pod:  &v1.Pod{},
-		},
 		{
 			name: "missing labels",
-			pod: &v1.Pod{
-				Spec: v1.PodSpec{
-					NodeSelector: map[string]string{
-						"foo": "bar",
-					},
-				},
-			},
-			wantStatus: framework.NewStatus(framework.UnschedulableAndUnresolvable, ErrReasonPod),
+			pod: st.MakePod().NodeSelector(map[string]string{
+				"foo": "bar",
+			}).Obj(),
+			wantStatus:   framework.NewStatus(framework.UnschedulableAndUnresolvable, ErrReasonPod),
+			runPreFilter: true,
 		},
 		{
 			name: "same labels",
-			pod: &v1.Pod{
-				Spec: v1.PodSpec{
-					NodeSelector: map[string]string{
-						"foo": "bar",
-					},
-				},
-			},
+			pod: st.MakePod().NodeSelector(map[string]string{
+				"foo": "bar",
+			}).Obj(),
 			labels: map[string]string{
 				"foo": "bar",
 			},
+			runPreFilter: true,
 		},
 		{
 			name: "node labels are superset",
-			pod: &v1.Pod{
-				Spec: v1.PodSpec{
-					NodeSelector: map[string]string{
-						"foo": "bar",
-					},
-				},
-			},
+			pod: st.MakePod().NodeSelector(map[string]string{
+				"foo": "bar",
+			}).Obj(),
 			labels: map[string]string{
 				"foo": "bar",
 				"baz": "blah",
 			},
+			runPreFilter: true,
 		},
 		{
 			name: "node labels are subset",
-			pod: &v1.Pod{
-				Spec: v1.PodSpec{
-					NodeSelector: map[string]string{
-						"foo": "bar",
-						"baz": "blah",
-					},
-				},
-			},
+			pod: st.MakePod().NodeSelector(map[string]string{
+				"foo": "bar",
+				"baz": "blah",
+			}).Obj(),
 			labels: map[string]string{
 				"foo": "bar",
 			},
-			wantStatus: framework.NewStatus(framework.UnschedulableAndUnresolvable, ErrReasonPod),
+			wantStatus:   framework.NewStatus(framework.UnschedulableAndUnresolvable, ErrReasonPod),
+			runPreFilter: true,
 		},
 		{
 			name: "Pod with matchExpressions using In operator that matches the existing node",
@@ -124,6 +115,7 @@ func TestNodeAffinity(t *testing.T) {
 			labels: map[string]string{
 				"foo": "bar",
 			},
+			runPreFilter: true,
 		},
 		{
 			name: "Pod with matchExpressions using Gt operator that matches the existing node",
@@ -152,6 +144,7 @@ func TestNodeAffinity(t *testing.T) {
 				// We use two digit to denote major version and two digit for minor version.
 				"kernel-version": "0206",
 			},
+			runPreFilter: true,
 		},
 		{
 			name: "Pod with matchExpressions using NotIn operator that matches the existing node",
@@ -179,6 +172,7 @@ func TestNodeAffinity(t *testing.T) {
 			labels: map[string]string{
 				"mem-type": "DDR3",
 			},
+			runPreFilter: true,
 		},
 		{
 			name: "Pod with matchExpressions using Exists operator that matches the existing node",
@@ -205,6 +199,7 @@ func TestNodeAffinity(t *testing.T) {
 			labels: map[string]string{
 				"GPU": "NVIDIA-GRID-K1",
 			},
+			runPreFilter: true,
 		},
 		{
 			name: "Pod with affinity that don't match node's labels won't schedule onto the node",
@@ -232,43 +227,8 @@ func TestNodeAffinity(t *testing.T) {
 			labels: map[string]string{
 				"foo": "bar",
 			},
-			wantStatus: framework.NewStatus(framework.UnschedulableAndUnresolvable, ErrReasonPod),
-		},
-		{
-			name: "Pod with a nil []NodeSelectorTerm in affinity, can't match the node's labels and won't schedule onto the node",
-			pod: &v1.Pod{
-				Spec: v1.PodSpec{
-					Affinity: &v1.Affinity{
-						NodeAffinity: &v1.NodeAffinity{
-							RequiredDuringSchedulingIgnoredDuringExecution: &v1.NodeSelector{
-								NodeSelectorTerms: nil,
-							},
-						},
-					},
-				},
-			},
-			labels: map[string]string{
-				"foo": "bar",
-			},
-			wantStatus: framework.NewStatus(framework.UnschedulableAndUnresolvable, ErrReasonPod),
-		},
-		{
-			name: "Pod with an empty []NodeSelectorTerm in affinity, can't match the node's labels and won't schedule onto the node",
-			pod: &v1.Pod{
-				Spec: v1.PodSpec{
-					Affinity: &v1.Affinity{
-						NodeAffinity: &v1.NodeAffinity{
-							RequiredDuringSchedulingIgnoredDuringExecution: &v1.NodeSelector{
-								NodeSelectorTerms: []v1.NodeSelectorTerm{},
-							},
-						},
-					},
-				},
-			},
-			labels: map[string]string{
-				"foo": "bar",
-			},
-			wantStatus: framework.NewStatus(framework.UnschedulableAndUnresolvable, ErrReasonPod),
+			wantStatus:   framework.NewStatus(framework.UnschedulableAndUnresolvable, ErrReasonPod),
+			runPreFilter: true,
 		},
 		{
 			name: "Pod with empty MatchExpressions is not a valid value will match no objects and won't schedule onto the node",
@@ -290,7 +250,8 @@ func TestNodeAffinity(t *testing.T) {
 			labels: map[string]string{
 				"foo": "bar",
 			},
-			wantStatus: framework.NewStatus(framework.UnschedulableAndUnresolvable, ErrReasonPod),
+			wantStatus:   framework.NewStatus(framework.UnschedulableAndUnresolvable, ErrReasonPod),
+			runPreFilter: true,
 		},
 		{
 			name: "Pod with no Affinity will schedule onto a node",
@@ -298,6 +259,8 @@ func TestNodeAffinity(t *testing.T) {
 			labels: map[string]string{
 				"foo": "bar",
 			},
+			wantPreFilterStatus: framework.NewStatus(framework.Skip),
+			runPreFilter:        true,
 		},
 		{
 			name: "Pod with Affinity but nil NodeSelector will schedule onto a node",
@@ -313,6 +276,8 @@ func TestNodeAffinity(t *testing.T) {
 			labels: map[string]string{
 				"foo": "bar",
 			},
+			wantPreFilterStatus: framework.NewStatus(framework.Skip),
+			runPreFilter:        true,
 		},
 		{
 			name: "Pod with multiple matchExpressions ANDed that matches the existing node",
@@ -343,6 +308,7 @@ func TestNodeAffinity(t *testing.T) {
 			labels: map[string]string{
 				"GPU": "NVIDIA-GRID-K1",
 			},
+			runPreFilter: true,
 		},
 		{
 			name: "Pod with multiple matchExpressions ANDed that doesn't match the existing node",
@@ -373,7 +339,8 @@ func TestNodeAffinity(t *testing.T) {
 			labels: map[string]string{
 				"GPU": "NVIDIA-GRID-K1",
 			},
-			wantStatus: framework.NewStatus(framework.UnschedulableAndUnresolvable, ErrReasonPod),
+			wantStatus:   framework.NewStatus(framework.UnschedulableAndUnresolvable, ErrReasonPod),
+			runPreFilter: true,
 		},
 		{
 			name: "Pod with multiple NodeSelectorTerms ORed in affinity, matches the node's labels and will schedule onto the node",
@@ -410,6 +377,7 @@ func TestNodeAffinity(t *testing.T) {
 			labels: map[string]string{
 				"foo": "bar",
 			},
+			runPreFilter: true,
 		},
 		{
 			name: "Pod with an Affinity and a PodSpec.NodeSelector(the old thing that we are deprecating) " +
@@ -440,6 +408,7 @@ func TestNodeAffinity(t *testing.T) {
 			labels: map[string]string{
 				"foo": "bar",
 			},
+			runPreFilter: true,
 		},
 		{
 			name: "Pod with an Affinity matches node's labels but the PodSpec.NodeSelector(the old thing that we are deprecating) " +
@@ -470,7 +439,8 @@ func TestNodeAffinity(t *testing.T) {
 			labels: map[string]string{
 				"foo": "barrrrrr",
 			},
-			wantStatus: framework.NewStatus(framework.UnschedulableAndUnresolvable, ErrReasonPod),
+			wantStatus:   framework.NewStatus(framework.UnschedulableAndUnresolvable, ErrReasonPod),
+			runPreFilter: true,
 		},
 		{
 			name: "Pod with an invalid value in Affinity term won't be scheduled onto the node",
@@ -498,7 +468,8 @@ func TestNodeAffinity(t *testing.T) {
 			labels: map[string]string{
 				"foo": "bar",
 			},
-			wantStatus: framework.NewStatus(framework.UnschedulableAndUnresolvable, ErrReasonPod),
+			wantStatus:   framework.NewStatus(framework.UnschedulableAndUnresolvable, ErrReasonPod),
+			runPreFilter: true,
 		},
 		{
 			name: "Pod with matchFields using In operator that matches the existing node",
@@ -513,7 +484,7 @@ func TestNodeAffinity(t *testing.T) {
 											{
 												Key:      metav1.ObjectNameField,
 												Operator: v1.NodeSelectorOpIn,
-												Values:   []string{"node_1"},
+												Values:   []string{"node1"},
 											},
 										},
 									},
@@ -523,7 +494,9 @@ func TestNodeAffinity(t *testing.T) {
 					},
 				},
 			},
-			nodeName: "node_1",
+			nodeName:            "node1",
+			wantPreFilterResult: &framework.PreFilterResult{NodeNames: sets.New("node1")},
+			runPreFilter:        true,
 		},
 		{
 			name: "Pod with matchFields using In operator that does not match the existing node",
@@ -538,7 +511,7 @@ func TestNodeAffinity(t *testing.T) {
 											{
 												Key:      metav1.ObjectNameField,
 												Operator: v1.NodeSelectorOpIn,
-												Values:   []string{"node_1"},
+												Values:   []string{"node1"},
 											},
 										},
 									},
@@ -548,8 +521,10 @@ func TestNodeAffinity(t *testing.T) {
 					},
 				},
 			},
-			nodeName:   "node_2",
-			wantStatus: framework.NewStatus(framework.UnschedulableAndUnresolvable, ErrReasonPod),
+			nodeName:            "node2",
+			wantStatus:          framework.NewStatus(framework.UnschedulableAndUnresolvable, ErrReasonPod),
+			wantPreFilterResult: &framework.PreFilterResult{NodeNames: sets.New("node1")},
+			runPreFilter:        true,
 		},
 		{
 			name: "Pod with two terms: matchFields does not match, but matchExpressions matches",
@@ -564,7 +539,12 @@ func TestNodeAffinity(t *testing.T) {
 											{
 												Key:      metav1.ObjectNameField,
 												Operator: v1.NodeSelectorOpIn,
-												Values:   []string{"node_1"},
+												Values:   []string{"node1"},
+											},
+											{
+												Key:      metav1.ObjectNameField,
+												Operator: v1.NodeSelectorOpIn,
+												Values:   []string{"node2"},
 											},
 										},
 									},
@@ -583,8 +563,9 @@ func TestNodeAffinity(t *testing.T) {
 					},
 				},
 			},
-			nodeName: "node_2",
-			labels:   map[string]string{"foo": "bar"},
+			nodeName:     "node2",
+			labels:       map[string]string{"foo": "bar"},
+			runPreFilter: true,
 		},
 		{
 			name: "Pod with one term: matchFields does not match, but matchExpressions matches",
@@ -599,7 +580,7 @@ func TestNodeAffinity(t *testing.T) {
 											{
 												Key:      metav1.ObjectNameField,
 												Operator: v1.NodeSelectorOpIn,
-												Values:   []string{"node_1"},
+												Values:   []string{"node1"},
 											},
 										},
 										MatchExpressions: []v1.NodeSelectorRequirement{
@@ -616,9 +597,11 @@ func TestNodeAffinity(t *testing.T) {
 					},
 				},
 			},
-			nodeName:   "node_2",
-			labels:     map[string]string{"foo": "bar"},
-			wantStatus: framework.NewStatus(framework.UnschedulableAndUnresolvable, ErrReasonPod),
+			nodeName:            "node2",
+			labels:              map[string]string{"foo": "bar"},
+			wantPreFilterResult: &framework.PreFilterResult{NodeNames: sets.New("node1")},
+			wantStatus:          framework.NewStatus(framework.UnschedulableAndUnresolvable, ErrReasonPod),
+			runPreFilter:        true,
 		},
 		{
 			name: "Pod with one term: both matchFields and matchExpressions match",
@@ -633,7 +616,7 @@ func TestNodeAffinity(t *testing.T) {
 											{
 												Key:      metav1.ObjectNameField,
 												Operator: v1.NodeSelectorOpIn,
-												Values:   []string{"node_1"},
+												Values:   []string{"node1"},
 											},
 										},
 										MatchExpressions: []v1.NodeSelectorRequirement{
@@ -650,8 +633,10 @@ func TestNodeAffinity(t *testing.T) {
 					},
 				},
 			},
-			nodeName: "node_1",
-			labels:   map[string]string{"foo": "bar"},
+			nodeName:            "node1",
+			labels:              map[string]string{"foo": "bar"},
+			wantPreFilterResult: &framework.PreFilterResult{NodeNames: sets.New("node1")},
+			runPreFilter:        true,
 		},
 		{
 			name: "Pod with two terms: both matchFields and matchExpressions do not match",
@@ -666,7 +651,7 @@ func TestNodeAffinity(t *testing.T) {
 											{
 												Key:      metav1.ObjectNameField,
 												Operator: v1.NodeSelectorOpIn,
-												Values:   []string{"node_1"},
+												Values:   []string{"node1"},
 											},
 										},
 									},
@@ -685,9 +670,80 @@ func TestNodeAffinity(t *testing.T) {
 					},
 				},
 			},
-			nodeName:   "node_2",
-			labels:     map[string]string{"foo": "bar"},
-			wantStatus: framework.NewStatus(framework.UnschedulableAndUnresolvable, ErrReasonPod),
+			nodeName:     "node2",
+			labels:       map[string]string{"foo": "bar"},
+			wantStatus:   framework.NewStatus(framework.UnschedulableAndUnresolvable, ErrReasonPod),
+			runPreFilter: true,
+		},
+		{
+			name: "Pod with two terms of node.Name affinity",
+			pod: &v1.Pod{
+				Spec: v1.PodSpec{
+					Affinity: &v1.Affinity{
+						NodeAffinity: &v1.NodeAffinity{
+							RequiredDuringSchedulingIgnoredDuringExecution: &v1.NodeSelector{
+								NodeSelectorTerms: []v1.NodeSelectorTerm{
+									{
+										MatchFields: []v1.NodeSelectorRequirement{
+											{
+												Key:      metav1.ObjectNameField,
+												Operator: v1.NodeSelectorOpIn,
+												Values:   []string{"node1"},
+											},
+										},
+									},
+									{
+										MatchFields: []v1.NodeSelectorRequirement{
+											{
+												Key:      metav1.ObjectNameField,
+												Operator: v1.NodeSelectorOpIn,
+												Values:   []string{"node2"},
+											},
+										},
+									},
+								},
+							},
+						},
+					},
+				},
+			},
+			nodeName:            "node2",
+			wantPreFilterResult: &framework.PreFilterResult{NodeNames: sets.New("node1", "node2")},
+			runPreFilter:        true,
+		},
+		{
+			name: "Pod with two conflicting mach field requirements",
+			pod: &v1.Pod{
+				Spec: v1.PodSpec{
+					Affinity: &v1.Affinity{
+						NodeAffinity: &v1.NodeAffinity{
+							RequiredDuringSchedulingIgnoredDuringExecution: &v1.NodeSelector{
+								NodeSelectorTerms: []v1.NodeSelectorTerm{
+									{
+										MatchFields: []v1.NodeSelectorRequirement{
+											{
+												Key:      metav1.ObjectNameField,
+												Operator: v1.NodeSelectorOpIn,
+												Values:   []string{"node1"},
+											},
+											{
+												Key:      metav1.ObjectNameField,
+												Operator: v1.NodeSelectorOpIn,
+												Values:   []string{"node2"},
+											},
+										},
+									},
+								},
+							},
+						},
+					},
+				},
+			},
+			nodeName:            "node2",
+			labels:              map[string]string{"foo": "bar"},
+			wantPreFilterStatus: framework.NewStatus(framework.UnschedulableAndUnresolvable, errReasonConflict),
+			wantStatus:          framework.NewStatus(framework.UnschedulableAndUnresolvable, ErrReasonPod),
+			runPreFilter:        true,
 		},
 		{
 			name: "Matches added affinity and Pod's node affinity",
@@ -712,7 +768,7 @@ func TestNodeAffinity(t *testing.T) {
 					},
 				},
 			},
-			nodeName: "node_2",
+			nodeName: "node2",
 			labels:   map[string]string{"zone": "foo"},
 			args: config.NodeAffinityArgs{
 				AddedAffinity: &v1.NodeAffinity{
@@ -721,12 +777,13 @@ func TestNodeAffinity(t *testing.T) {
 							MatchFields: []v1.NodeSelectorRequirement{{
 								Key:      metav1.ObjectNameField,
 								Operator: v1.NodeSelectorOpIn,
-								Values:   []string{"node_2"},
+								Values:   []string{"node2"},
 							}},
 						}},
 					},
 				},
 			},
+			runPreFilter: true,
 		},
 		{
 			name: "Matches added affinity but not Pod's node affinity",
@@ -751,7 +808,7 @@ func TestNodeAffinity(t *testing.T) {
 					},
 				},
 			},
-			nodeName: "node_2",
+			nodeName: "node2",
 			labels:   map[string]string{"zone": "foo"},
 			args: config.NodeAffinityArgs{
 				AddedAffinity: &v1.NodeAffinity{
@@ -760,18 +817,19 @@ func TestNodeAffinity(t *testing.T) {
 							MatchFields: []v1.NodeSelectorRequirement{{
 								Key:      metav1.ObjectNameField,
 								Operator: v1.NodeSelectorOpIn,
-								Values:   []string{"node_2"},
+								Values:   []string{"node2"},
 							}},
 						}},
 					},
 				},
 			},
-			wantStatus: framework.NewStatus(framework.UnschedulableAndUnresolvable, ErrReasonPod),
+			wantStatus:   framework.NewStatus(framework.UnschedulableAndUnresolvable, ErrReasonPod),
+			runPreFilter: true,
 		},
 		{
 			name:     "Doesn't match added affinity",
 			pod:      &v1.Pod{},
-			nodeName: "node_2",
+			nodeName: "node2",
 			labels:   map[string]string{"zone": "foo"},
 			args: config.NodeAffinityArgs{
 				AddedAffinity: &v1.NodeAffinity{
@@ -788,7 +846,8 @@ func TestNodeAffinity(t *testing.T) {
 					},
 				},
 			},
-			wantStatus: framework.NewStatus(framework.UnschedulableAndUnresolvable, errReasonEnforced),
+			wantStatus:   framework.NewStatus(framework.UnschedulableAndUnresolvable, errReasonEnforced),
+			runPreFilter: true,
 		},
 		{
 			name: "Matches node selector correctly even if PreFilter is not called",
@@ -803,7 +862,7 @@ func TestNodeAffinity(t *testing.T) {
 				"foo": "bar",
 				"baz": "blah",
 			},
-			disablePreFilter: true,
+			runPreFilter: false,
 		},
 		{
 			name: "Matches node affinity correctly even if PreFilter is not called",
@@ -834,12 +893,13 @@ func TestNodeAffinity(t *testing.T) {
 			labels: map[string]string{
 				"GPU": "NVIDIA-GRID-K1",
 			},
-			disablePreFilter: true,
+			runPreFilter: false,
 		},
 	}
 
 	for _, test := range tests {
 		t.Run(test.name, func(t *testing.T) {
+			_, ctx := ktesting.NewTestContext(t)
 			node := v1.Node{ObjectMeta: metav1.ObjectMeta{
 				Name:   test.nodeName,
 				Labels: test.labels,
@@ -847,22 +907,25 @@ func TestNodeAffinity(t *testing.T) {
 			nodeInfo := framework.NewNodeInfo()
 			nodeInfo.SetNode(&node)
 
-			p, err := New(&test.args, nil)
+			p, err := New(ctx, &test.args, nil)
 			if err != nil {
 				t.Fatalf("Creating plugin: %v", err)
 			}
 
 			state := framework.NewCycleState()
 			var gotStatus *framework.Status
-			if !test.disablePreFilter {
-				gotStatus = p.(framework.PreFilterPlugin).PreFilter(context.Background(), state, test.pod)
-				if !gotStatus.IsSuccess() {
-					t.Errorf("unexpected error: %v", gotStatus)
+			if test.runPreFilter {
+				gotPreFilterResult, gotStatus := p.(framework.PreFilterPlugin).PreFilter(context.Background(), state, test.pod)
+				if diff := cmp.Diff(test.wantPreFilterStatus, gotStatus); diff != "" {
+					t.Errorf("unexpected PreFilter Status (-want,+got):\n%s", diff)
+				}
+				if diff := cmp.Diff(test.wantPreFilterResult, gotPreFilterResult); diff != "" {
+					t.Errorf("unexpected PreFilterResult (-want,+got):\n%s", diff)
 				}
 			}
 			gotStatus = p.(framework.FilterPlugin).Filter(context.Background(), state, test.pod, nodeInfo)
-			if !reflect.DeepEqual(gotStatus, test.wantStatus) {
-				t.Errorf("status does not match: %v, want: %v", gotStatus, test.wantStatus)
+			if diff := cmp.Diff(test.wantStatus, gotStatus); diff != "" {
+				t.Errorf("unexpected Filter Status (-want,+got):\n%s", diff)
 			}
 		})
 	}
@@ -944,80 +1007,132 @@ func TestNodeAffinityPriority(t *testing.T) {
 	}
 
 	tests := []struct {
-		name            string
-		pod             *v1.Pod
-		nodes           []*v1.Node
-		expectedList    framework.NodeScoreList
-		args            config.NodeAffinityArgs
-		disablePreScore bool
+		name               string
+		pod                *v1.Pod
+		nodes              []*v1.Node
+		expectedList       framework.NodeScoreList
+		args               config.NodeAffinityArgs
+		runPreScore        bool
+		wantPreScoreStatus *framework.Status
 	}{
 		{
-			name: "all machines are same priority as NodeAffinity is nil",
+			name: "all nodes are same priority as NodeAffinity is nil",
 			pod: &v1.Pod{
 				ObjectMeta: metav1.ObjectMeta{
 					Annotations: map[string]string{},
 				},
 			},
 			nodes: []*v1.Node{
-				{ObjectMeta: metav1.ObjectMeta{Name: "machine1", Labels: label1}},
-				{ObjectMeta: metav1.ObjectMeta{Name: "machine2", Labels: label2}},
-				{ObjectMeta: metav1.ObjectMeta{Name: "machine3", Labels: label3}},
+				{ObjectMeta: metav1.ObjectMeta{Name: "node1", Labels: label1}},
+				{ObjectMeta: metav1.ObjectMeta{Name: "node2", Labels: label2}},
+				{ObjectMeta: metav1.ObjectMeta{Name: "node3", Labels: label3}},
 			},
-			expectedList: []framework.NodeScore{{Name: "machine1", Score: 0}, {Name: "machine2", Score: 0}, {Name: "machine3", Score: 0}},
+			expectedList: []framework.NodeScore{{Name: "node1", Score: 0}, {Name: "node2", Score: 0}, {Name: "node3", Score: 0}},
 		},
 		{
-			name: "no machine matches preferred scheduling requirements in NodeAffinity of pod so all machines' priority is zero",
+			// PreScore returns Skip.
+			name: "Skip is returned in PreScore when NodeAffinity is nil",
+			pod: &v1.Pod{
+				ObjectMeta: metav1.ObjectMeta{
+					Annotations: map[string]string{},
+				},
+			},
+			nodes: []*v1.Node{
+				{ObjectMeta: metav1.ObjectMeta{Name: "node1", Labels: label1}},
+			},
+			runPreScore:        true,
+			wantPreScoreStatus: framework.NewStatus(framework.Skip),
+		},
+		{
+			name: "PreScore returns error when an incoming Pod has a broken affinity",
+			pod: &v1.Pod{
+				ObjectMeta: metav1.ObjectMeta{
+					Annotations: map[string]string{},
+				},
+				Spec: v1.PodSpec{
+					Affinity: &v1.Affinity{
+						NodeAffinity: &v1.NodeAffinity{
+							PreferredDuringSchedulingIgnoredDuringExecution: []v1.PreferredSchedulingTerm{
+								{
+									Weight: 2,
+									Preference: v1.NodeSelectorTerm{
+										MatchExpressions: []v1.NodeSelectorRequirement{
+											{
+												Key:      "invalid key",
+												Operator: v1.NodeSelectorOpIn,
+												Values:   []string{"bar"},
+											},
+										},
+									},
+								},
+							},
+						},
+					},
+				},
+			},
+			nodes: []*v1.Node{
+				{ObjectMeta: metav1.ObjectMeta{Name: "node1", Labels: label1}},
+			},
+			runPreScore:        true,
+			wantPreScoreStatus: framework.AsStatus(fmt.Errorf(`[0].matchExpressions[0].key: Invalid value: "invalid key": name part must consist of alphanumeric characters, '-', '_' or '.', and must start and end with an alphanumeric character (e.g. 'MyName',  or 'my.name',  or '123-abc', regex used for validation is '([A-Za-z0-9][-A-Za-z0-9_.]*)?[A-Za-z0-9]')`)),
+		},
+		{
+			name: "no node matches preferred scheduling requirements in NodeAffinity of pod so all nodes' priority is zero",
 			pod: &v1.Pod{
 				Spec: v1.PodSpec{
 					Affinity: affinity1,
 				},
 			},
 			nodes: []*v1.Node{
-				{ObjectMeta: metav1.ObjectMeta{Name: "machine1", Labels: label4}},
-				{ObjectMeta: metav1.ObjectMeta{Name: "machine2", Labels: label2}},
-				{ObjectMeta: metav1.ObjectMeta{Name: "machine3", Labels: label3}},
+				{ObjectMeta: metav1.ObjectMeta{Name: "node1", Labels: label4}},
+				{ObjectMeta: metav1.ObjectMeta{Name: "node2", Labels: label2}},
+				{ObjectMeta: metav1.ObjectMeta{Name: "node3", Labels: label3}},
 			},
-			expectedList: []framework.NodeScore{{Name: "machine1", Score: 0}, {Name: "machine2", Score: 0}, {Name: "machine3", Score: 0}},
+			expectedList: []framework.NodeScore{{Name: "node1", Score: 0}, {Name: "node2", Score: 0}, {Name: "node3", Score: 0}},
+			runPreScore:  true,
 		},
 		{
-			name: "only machine1 matches the preferred scheduling requirements of pod",
+			name: "only node1 matches the preferred scheduling requirements of pod",
 			pod: &v1.Pod{
 				Spec: v1.PodSpec{
 					Affinity: affinity1,
 				},
 			},
 			nodes: []*v1.Node{
-				{ObjectMeta: metav1.ObjectMeta{Name: "machine1", Labels: label1}},
-				{ObjectMeta: metav1.ObjectMeta{Name: "machine2", Labels: label2}},
-				{ObjectMeta: metav1.ObjectMeta{Name: "machine3", Labels: label3}},
+				{ObjectMeta: metav1.ObjectMeta{Name: "node1", Labels: label1}},
+				{ObjectMeta: metav1.ObjectMeta{Name: "node2", Labels: label2}},
+				{ObjectMeta: metav1.ObjectMeta{Name: "node3", Labels: label3}},
 			},
-			expectedList: []framework.NodeScore{{Name: "machine1", Score: framework.MaxNodeScore}, {Name: "machine2", Score: 0}, {Name: "machine3", Score: 0}},
+			expectedList: []framework.NodeScore{{Name: "node1", Score: framework.MaxNodeScore}, {Name: "node2", Score: 0}, {Name: "node3", Score: 0}},
+			runPreScore:  true,
 		},
 		{
-			name: "all machines matches the preferred scheduling requirements of pod but with different priorities ",
+			name: "all nodes matches the preferred scheduling requirements of pod but with different priorities ",
 			pod: &v1.Pod{
 				Spec: v1.PodSpec{
 					Affinity: affinity2,
 				},
 			},
 			nodes: []*v1.Node{
-				{ObjectMeta: metav1.ObjectMeta{Name: "machine1", Labels: label1}},
-				{ObjectMeta: metav1.ObjectMeta{Name: "machine5", Labels: label5}},
-				{ObjectMeta: metav1.ObjectMeta{Name: "machine2", Labels: label2}},
+				{ObjectMeta: metav1.ObjectMeta{Name: "node1", Labels: label1}},
+				{ObjectMeta: metav1.ObjectMeta{Name: "node5", Labels: label5}},
+				{ObjectMeta: metav1.ObjectMeta{Name: "node2", Labels: label2}},
 			},
-			expectedList: []framework.NodeScore{{Name: "machine1", Score: 18}, {Name: "machine5", Score: framework.MaxNodeScore}, {Name: "machine2", Score: 36}},
+			expectedList: []framework.NodeScore{{Name: "node1", Score: 18}, {Name: "node5", Score: framework.MaxNodeScore}, {Name: "node2", Score: 36}},
+			runPreScore:  true,
 		},
 		{
 			name: "added affinity",
 			pod:  &v1.Pod{},
 			nodes: []*v1.Node{
-				{ObjectMeta: metav1.ObjectMeta{Name: "machine1", Labels: label1}},
-				{ObjectMeta: metav1.ObjectMeta{Name: "machine2", Labels: label2}},
+				{ObjectMeta: metav1.ObjectMeta{Name: "node1", Labels: label1}},
+				{ObjectMeta: metav1.ObjectMeta{Name: "node2", Labels: label2}},
 			},
-			expectedList: []framework.NodeScore{{Name: "machine1", Score: framework.MaxNodeScore}, {Name: "machine2", Score: 0}},
+			expectedList: []framework.NodeScore{{Name: "node1", Score: framework.MaxNodeScore}, {Name: "node2", Score: 0}},
 			args: config.NodeAffinityArgs{
 				AddedAffinity: affinity1.NodeAffinity,
 			},
+			runPreScore: true,
 		},
 		{
 			name: "added affinity and pod has default affinity",
@@ -1027,11 +1142,11 @@ func TestNodeAffinityPriority(t *testing.T) {
 				},
 			},
 			nodes: []*v1.Node{
-				{ObjectMeta: metav1.ObjectMeta{Name: "machine1", Labels: label1}},
-				{ObjectMeta: metav1.ObjectMeta{Name: "machine2", Labels: label2}},
-				{ObjectMeta: metav1.ObjectMeta{Name: "machine3", Labels: label5}},
+				{ObjectMeta: metav1.ObjectMeta{Name: "node1", Labels: label1}},
+				{ObjectMeta: metav1.ObjectMeta{Name: "node2", Labels: label2}},
+				{ObjectMeta: metav1.ObjectMeta{Name: "node3", Labels: label5}},
 			},
-			expectedList: []framework.NodeScore{{Name: "machine1", Score: 40}, {Name: "machine2", Score: 60}, {Name: "machine3", Score: framework.MaxNodeScore}},
+			expectedList: []framework.NodeScore{{Name: "node1", Score: 40}, {Name: "node2", Score: 60}, {Name: "node3", Score: framework.MaxNodeScore}},
 			args: config.NodeAffinityArgs{
 				AddedAffinity: &v1.NodeAffinity{
 					PreferredDuringSchedulingIgnoredDuringExecution: []v1.PreferredSchedulingTerm{
@@ -1050,6 +1165,7 @@ func TestNodeAffinityPriority(t *testing.T) {
 					},
 				},
 			},
+			runPreScore: true,
 		},
 		{
 			name: "calculate the priorities correctly even if PreScore is not called",
@@ -1059,41 +1175,52 @@ func TestNodeAffinityPriority(t *testing.T) {
 				},
 			},
 			nodes: []*v1.Node{
-				{ObjectMeta: metav1.ObjectMeta{Name: "machine1", Labels: label1}},
-				{ObjectMeta: metav1.ObjectMeta{Name: "machine5", Labels: label5}},
-				{ObjectMeta: metav1.ObjectMeta{Name: "machine2", Labels: label2}},
+				{ObjectMeta: metav1.ObjectMeta{Name: "node1", Labels: label1}},
+				{ObjectMeta: metav1.ObjectMeta{Name: "node5", Labels: label5}},
+				{ObjectMeta: metav1.ObjectMeta{Name: "node2", Labels: label2}},
 			},
-			expectedList:    []framework.NodeScore{{Name: "machine1", Score: 18}, {Name: "machine5", Score: framework.MaxNodeScore}, {Name: "machine2", Score: 36}},
-			disablePreScore: true,
+			expectedList: []framework.NodeScore{{Name: "node1", Score: 18}, {Name: "node5", Score: framework.MaxNodeScore}, {Name: "node2", Score: 36}},
+			runPreScore:  true,
 		},
 	}
 
 	for _, test := range tests {
 		t.Run(test.name, func(t *testing.T) {
+			_, ctx := ktesting.NewTestContext(t)
+			ctx, cancel := context.WithCancel(ctx)
+			defer cancel()
+
 			state := framework.NewCycleState()
-			fh, _ := runtime.NewFramework(nil, nil, runtime.WithSnapshotSharedLister(cache.NewSnapshot(nil, test.nodes)))
-			p, err := New(&test.args, fh)
+			fh, _ := runtime.NewFramework(ctx, nil, nil, runtime.WithSnapshotSharedLister(cache.NewSnapshot(nil, test.nodes)))
+			p, err := New(ctx, &test.args, fh)
 			if err != nil {
 				t.Fatalf("Creating plugin: %v", err)
 			}
 			var status *framework.Status
-			if !test.disablePreScore {
-				status = p.(framework.PreScorePlugin).PreScore(context.Background(), state, test.pod, test.nodes)
+			if test.runPreScore {
+				status = p.(framework.PreScorePlugin).PreScore(ctx, state, test.pod, tf.BuildNodeInfos(test.nodes))
+				if status.Code() != test.wantPreScoreStatus.Code() {
+					t.Errorf("unexpected status code from PreScore: want: %v got: %v", test.wantPreScoreStatus.Code().String(), status.Code().String())
+				}
+				if status.Message() != test.wantPreScoreStatus.Message() {
+					t.Errorf("unexpected status message from PreScore: want: %v got: %v", test.wantPreScoreStatus.Message(), status.Message())
+				}
 				if !status.IsSuccess() {
-					t.Errorf("unexpected error: %v", status)
+					// no need to proceed.
+					return
 				}
 			}
 			var gotList framework.NodeScoreList
 			for _, n := range test.nodes {
 				nodeName := n.ObjectMeta.Name
-				score, status := p.(framework.ScorePlugin).Score(context.Background(), state, test.pod, nodeName)
+				score, status := p.(framework.ScorePlugin).Score(ctx, state, test.pod, nodeName)
 				if !status.IsSuccess() {
 					t.Errorf("unexpected error: %v", status)
 				}
 				gotList = append(gotList, framework.NodeScore{Name: nodeName, Score: score})
 			}
 
-			status = p.(framework.ScorePlugin).ScoreExtensions().NormalizeScore(context.Background(), state, test.pod, gotList)
+			status = p.(framework.ScorePlugin).ScoreExtensions().NormalizeScore(ctx, state, test.pod, gotList)
 			if !status.IsSuccess() {
 				t.Errorf("unexpected error: %v", status)
 			}
@@ -1104,3 +1231,132 @@ func TestNodeAffinityPriority(t *testing.T) {
 		})
 	}
 }
+
+func Test_isSchedulableAfterNodeChange(t *testing.T) {
+	podWithNodeAffinity := st.MakePod().NodeAffinityIn("foo", []string{"bar"})
+	testcases := map[string]struct {
+		args           *config.NodeAffinityArgs
+		pod            *v1.Pod
+		oldObj, newObj interface{}
+		expectedHint   framework.QueueingHint
+		expectedErr    bool
+	}{
+		"backoff-wrong-new-object": {
+			args:         &config.NodeAffinityArgs{},
+			pod:          podWithNodeAffinity.Obj(),
+			newObj:       "not-a-node",
+			expectedHint: framework.Queue,
+			expectedErr:  true,
+		},
+		"backoff-wrong-old-object": {
+			args:         &config.NodeAffinityArgs{},
+			pod:          podWithNodeAffinity.Obj(),
+			oldObj:       "not-a-node",
+			newObj:       st.MakeNode().Obj(),
+			expectedHint: framework.Queue,
+			expectedErr:  true,
+		},
+		"skip-queue-on-add": {
+			args:         &config.NodeAffinityArgs{},
+			pod:          podWithNodeAffinity.Obj(),
+			newObj:       st.MakeNode().Obj(),
+			expectedHint: framework.QueueSkip,
+		},
+		"queue-on-add": {
+			args:         &config.NodeAffinityArgs{},
+			pod:          podWithNodeAffinity.Obj(),
+			newObj:       st.MakeNode().Label("foo", "bar").Obj(),
+			expectedHint: framework.Queue,
+		},
+		"skip-unrelated-changes": {
+			args:         &config.NodeAffinityArgs{},
+			pod:          podWithNodeAffinity.Obj(),
+			oldObj:       st.MakeNode().Obj(),
+			newObj:       st.MakeNode().Capacity(nil).Obj(),
+			expectedHint: framework.QueueSkip,
+		},
+		"skip-unrelated-changes-on-labels": {
+			args:         &config.NodeAffinityArgs{},
+			pod:          podWithNodeAffinity.DeepCopy(),
+			oldObj:       st.MakeNode().Obj(),
+			newObj:       st.MakeNode().Label("k", "v").Obj(),
+			expectedHint: framework.QueueSkip,
+		},
+		"skip-labels-changes-on-node-from-suitable-to-unsuitable": {
+			args:         &config.NodeAffinityArgs{},
+			pod:          podWithNodeAffinity.DeepCopy(),
+			oldObj:       st.MakeNode().Label("foo", "bar").Obj(),
+			newObj:       st.MakeNode().Label("k", "v").Obj(),
+			expectedHint: framework.QueueSkip,
+		},
+		"queue-on-labels-change-makes-pod-schedulable": {
+			args:         &config.NodeAffinityArgs{},
+			pod:          podWithNodeAffinity.Obj(),
+			oldObj:       st.MakeNode().Obj(),
+			newObj:       st.MakeNode().Label("foo", "bar").Obj(),
+			expectedHint: framework.Queue,
+		},
+		"skip-queue-on-add-scheduler-enforced-node-affinity": {
+			args: &config.NodeAffinityArgs{
+				AddedAffinity: &v1.NodeAffinity{
+					RequiredDuringSchedulingIgnoredDuringExecution: &v1.NodeSelector{
+						NodeSelectorTerms: []v1.NodeSelectorTerm{
+							{
+								MatchExpressions: []v1.NodeSelectorRequirement{
+									{
+										Key:      "foo",
+										Operator: v1.NodeSelectorOpIn,
+										Values:   []string{"bar"},
+									},
+								},
+							},
+						},
+					},
+				},
+			},
+			pod:          podWithNodeAffinity.Obj(),
+			newObj:       st.MakeNode().Obj(),
+			expectedHint: framework.QueueSkip,
+		},
+		"queue-on-add-scheduler-enforced-node-affinity": {
+			args: &config.NodeAffinityArgs{
+				AddedAffinity: &v1.NodeAffinity{
+					RequiredDuringSchedulingIgnoredDuringExecution: &v1.NodeSelector{
+						NodeSelectorTerms: []v1.NodeSelectorTerm{
+							{
+								MatchExpressions: []v1.NodeSelectorRequirement{
+									{
+										Key:      "foo",
+										Operator: v1.NodeSelectorOpIn,
+										Values:   []string{"bar"},
+									},
+								},
+							},
+						},
+					},
+				},
+			},
+			pod:          podWithNodeAffinity.Obj(),
+			newObj:       st.MakeNode().Label("foo", "bar").Obj(),
+			expectedHint: framework.Queue,
+		},
+	}
+
+	for name, tc := range testcases {
+		t.Run(name, func(t *testing.T) {
+			logger, ctx := ktesting.NewTestContext(t)
+			p, err := New(ctx, tc.args, nil)
+			if err != nil {
+				t.Fatalf("Creating plugin: %v", err)
+			}
+
+			actualHint, err := p.(*NodeAffinity).isSchedulableAfterNodeChange(logger, tc.pod, tc.oldObj, tc.newObj)
+			if tc.expectedErr {
+				require.Error(t, err)
+				return
+			}
+			require.NoError(t, err)
+			require.Equal(t, tc.expectedHint, actualHint)
+		})
+	}
+}
