@@ -28,6 +28,7 @@ import (
 	"k8s.io/client-go/kubernetes/fake"
 	"k8s.io/client-go/kubernetes/scheme"
 	"k8s.io/klog/v2"
+	"k8s.io/klog/v2/ktesting"
 
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/runtime"
@@ -44,135 +45,162 @@ import (
 	"github.com/stretchr/testify/assert"
 )
 
-func TestEnsureNodeExistsByProviderID(t *testing.T) {
-
-	testCases := []struct {
-		testName           string
-		node               *v1.Node
-		expectedCalls      []string
-		expectedNodeExists bool
-		hasInstanceID      bool
-		existsByProviderID bool
-		nodeNameErr        error
-		providerIDErr      error
-	}{
-		{
-			testName:           "node exists by provider id",
-			existsByProviderID: true,
-			providerIDErr:      nil,
-			hasInstanceID:      true,
-			nodeNameErr:        errors.New("unimplemented"),
-			expectedCalls:      []string{"instance-exists-by-provider-id"},
-			expectedNodeExists: true,
-			node: &v1.Node{
-				ObjectMeta: metav1.ObjectMeta{
-					Name: "node0",
-				},
-				Spec: v1.NodeSpec{
-					ProviderID: "node0",
-				},
-			},
-		},
-		{
-			testName:           "does not exist by provider id",
-			existsByProviderID: false,
-			providerIDErr:      nil,
-			hasInstanceID:      true,
-			nodeNameErr:        errors.New("unimplemented"),
-			expectedCalls:      []string{"instance-exists-by-provider-id"},
-			expectedNodeExists: false,
-			node: &v1.Node{
-				ObjectMeta: metav1.ObjectMeta{
-					Name: "node0",
-				},
-				Spec: v1.NodeSpec{
-					ProviderID: "node0",
-				},
-			},
-		},
-		{
-			testName:           "exists by instance id",
-			existsByProviderID: true,
-			providerIDErr:      nil,
-			hasInstanceID:      true,
-			nodeNameErr:        nil,
-			expectedCalls:      []string{"instance-id", "instance-exists-by-provider-id"},
-			expectedNodeExists: true,
-			node: &v1.Node{
-				ObjectMeta: metav1.ObjectMeta{
-					Name: "node0",
-				},
-			},
-		},
-		{
-			testName:           "does not exist by no instance id",
-			existsByProviderID: true,
-			providerIDErr:      nil,
-			hasInstanceID:      false,
-			nodeNameErr:        cloudprovider.InstanceNotFound,
-			expectedCalls:      []string{"instance-id"},
-			expectedNodeExists: false,
-			node: &v1.Node{
-				ObjectMeta: metav1.ObjectMeta{
-					Name: "node0",
-				},
-			},
-		},
-		{
-			testName:           "provider id returns error",
-			existsByProviderID: false,
-			providerIDErr:      errors.New("unimplemented"),
-			hasInstanceID:      true,
-			nodeNameErr:        cloudprovider.InstanceNotFound,
-			expectedCalls:      []string{"instance-exists-by-provider-id"},
-			expectedNodeExists: false,
-			node: &v1.Node{
-				ObjectMeta: metav1.ObjectMeta{
-					Name: "node0",
-				},
-				Spec: v1.NodeSpec{
-					ProviderID: "node0",
-				},
-			},
-		},
-	}
-
-	for _, tc := range testCases {
-		t.Run(tc.testName, func(t *testing.T) {
-			fc := &fakecloud.Cloud{
-				ExistsByProviderID: tc.existsByProviderID,
-				Err:                tc.nodeNameErr,
-				ErrByProviderID:    tc.providerIDErr,
-			}
-
-			if tc.hasInstanceID {
-				fc.ExtID = map[types.NodeName]string{
-					types.NodeName(tc.node.Name): "provider-id://a",
-				}
-			}
-
-			instances, _ := fc.Instances()
-			exists, err := ensureNodeExistsByProviderID(context.TODO(), instances, tc.node)
-			assert.Equal(t, err, tc.providerIDErr)
-
-			assert.EqualValues(t, tc.expectedCalls, fc.Calls,
-				"expected cloud provider methods `%v` to be called but `%v` was called ",
-				tc.expectedCalls, fc.Calls)
-
-			assert.Equal(t, tc.expectedNodeExists, exists,
-				"expected exists to be `%t` but got `%t`",
-				tc.existsByProviderID, exists)
-		})
-	}
-}
-
 func Test_syncNode(t *testing.T) {
 	tests := []struct {
 		name         string
 		fakeCloud    *fakecloud.Cloud
 		existingNode *v1.Node
 		updatedNode  *v1.Node
+		expectedErr  bool
 	}{
+		{
+			name: "node initialized with hostname",
+			fakeCloud: &fakecloud.Cloud{
+				EnableInstancesV2: false,
+				InstanceTypes: map[types.NodeName]string{
+					types.NodeName("node0"): "t1.micro",
+				},
+				ExtID: map[types.NodeName]string{
+					types.NodeName("node0"): "12345",
+				},
+				Addresses: []v1.NodeAddress{
+					{
+						Type:    v1.NodeHostName,
+						Address: "node0.cloud.internal",
+					},
+					{
+						Type:    v1.NodeInternalIP,
+						Address: "10.0.0.1",
+					},
+					{
+						Type:    v1.NodeExternalIP,
+						Address: "132.143.154.163",
+					},
+				},
+				ErrByProviderID: nil,
+				Err:             nil,
+			},
+			existingNode: &v1.Node{
+				ObjectMeta: metav1.ObjectMeta{
+					Name:              "node0",
+					CreationTimestamp: metav1.Date(2012, 1, 1, 0, 0, 0, 0, time.UTC),
+				},
+				Spec: v1.NodeSpec{
+					Taints: []v1.Taint{
+						{
+							Key:    cloudproviderapi.TaintExternalCloudProvider,
+							Value:  "true",
+							Effect: v1.TaintEffectNoSchedule,
+						},
+					},
+				},
+				Status: v1.NodeStatus{
+					Addresses: []v1.NodeAddress{
+						{
+							Type:    v1.NodeHostName,
+							Address: "nodeoverriden",
+						},
+					},
+				},
+			},
+			updatedNode: &v1.Node{
+				ObjectMeta: metav1.ObjectMeta{
+					Name:              "node0",
+					CreationTimestamp: metav1.Date(2012, 1, 1, 0, 0, 0, 0, time.UTC),
+				},
+				Spec: v1.NodeSpec{
+					ProviderID: "fake://12345",
+				},
+				Status: v1.NodeStatus{
+					Addresses: []v1.NodeAddress{
+						{
+							Type:    v1.NodeHostName,
+							Address: "node0.cloud.internal",
+						},
+						{
+							Type:    v1.NodeInternalIP,
+							Address: "10.0.0.1",
+						},
+						{
+							Type:    v1.NodeExternalIP,
+							Address: "132.143.154.163",
+						},
+					},
+				},
+			},
+		},
+		{
+			name: "node initialized with hostname and cloud provider missing hostname",
+			fakeCloud: &fakecloud.Cloud{
+				EnableInstancesV2: false,
+				InstanceTypes: map[types.NodeName]string{
+					types.NodeName("node0"): "t1.micro",
+				},
+				ExtID: map[types.NodeName]string{
+					types.NodeName("node0"): "12345",
+				},
+				Addresses: []v1.NodeAddress{
+					{
+						Type:    v1.NodeInternalIP,
+						Address: "10.0.0.1",
+					},
+					{
+						Type:    v1.NodeExternalIP,
+						Address: "132.143.154.163",
+					},
+				},
+				ErrByProviderID: nil,
+				Err:             nil,
+			},
+			existingNode: &v1.Node{
+				ObjectMeta: metav1.ObjectMeta{
+					Name:              "node0",
+					CreationTimestamp: metav1.Date(2012, 1, 1, 0, 0, 0, 0, time.UTC),
+				},
+				Spec: v1.NodeSpec{
+					Taints: []v1.Taint{
+						{
+							Key:    cloudproviderapi.TaintExternalCloudProvider,
+							Value:  "true",
+							Effect: v1.TaintEffectNoSchedule,
+						},
+					},
+				},
+				Status: v1.NodeStatus{
+					Addresses: []v1.NodeAddress{
+						{
+							Type:    v1.NodeHostName,
+							Address: "nodeoverriden",
+						},
+					},
+				},
+			},
+			updatedNode: &v1.Node{
+				ObjectMeta: metav1.ObjectMeta{
+					Name:              "node0",
+					CreationTimestamp: metav1.Date(2012, 1, 1, 0, 0, 0, 0, time.UTC),
+				},
+				Spec: v1.NodeSpec{
+					ProviderID: "fake://12345",
+				},
+				Status: v1.NodeStatus{
+					Addresses: []v1.NodeAddress{
+						{
+							Type:    v1.NodeInternalIP,
+							Address: "10.0.0.1",
+						},
+						{
+							Type:    v1.NodeExternalIP,
+							Address: "132.143.154.163",
+						},
+						{
+							Type:    v1.NodeHostName,
+							Address: "nodeoverriden",
+						},
+					},
+				},
+			},
+		},
 		{
 			name: "node initialized with provider ID",
 			fakeCloud: &fakecloud.Cloud{
@@ -667,7 +695,8 @@ func Test_syncNode(t *testing.T) {
 			},
 		},
 		{
-			name: "provided node IP address is not valid",
+			name:        "provided node IP address is not valid",
+			expectedErr: true,
 			fakeCloud: &fakecloud.Cloud{
 				EnableInstancesV2: false,
 				Addresses: []v1.NodeAddress{
@@ -765,7 +794,8 @@ func Test_syncNode(t *testing.T) {
 			},
 		},
 		{
-			name: "provided node IP address is not present",
+			name:        "provided node IP address is not present",
+			expectedErr: true,
 			fakeCloud: &fakecloud.Cloud{
 				EnableInstancesV2: false,
 				Addresses: []v1.NodeAddress{
@@ -957,8 +987,10 @@ func Test_syncNode(t *testing.T) {
 				},
 			},
 		},
-		{
-			name: "provider ID not implemented",
+		{ // for backward compatibility the cloud providers that does not implement
+			// providerID does not block the node initialization
+			name:        "provider ID not implemented",
+			expectedErr: false,
 			fakeCloud: &fakecloud.Cloud{
 				EnableInstancesV2: false,
 				InstanceTypes:     map[types.NodeName]string{},
@@ -1424,6 +1456,249 @@ func Test_syncNode(t *testing.T) {
 				},
 			},
 		},
+		{
+			name: "[instanceV2] provided additional labels",
+			fakeCloud: &fakecloud.Cloud{
+				EnableInstancesV2: true,
+				Addresses: []v1.NodeAddress{
+					{
+						Type:    v1.NodeInternalIP,
+						Address: "10.0.0.1",
+					},
+					{
+						Type:    v1.NodeExternalIP,
+						Address: "132.143.154.163",
+					},
+				},
+				ExistsByProviderID: true,
+				Err:                nil,
+				Zone: cloudprovider.Zone{
+					FailureDomain: "us-west-1a",
+					Region:        "us-west",
+				},
+				AdditionalLabels: map[string]string{
+					"topology.k8s.cp/zone-id": "az1",
+					"my.custom.label/foo":     "bar",
+				},
+			},
+			existingNode: &v1.Node{
+				ObjectMeta: metav1.ObjectMeta{
+					Name:              "node0",
+					CreationTimestamp: metav1.Date(2012, 1, 1, 0, 0, 0, 0, time.UTC),
+					Annotations: map[string]string{
+						cloudproviderapi.AnnotationAlphaProvidedIPAddr: "10.0.0.1",
+					},
+				},
+				Spec: v1.NodeSpec{
+					Taints: []v1.Taint{
+						{
+							Key:    "ImproveCoverageTaint",
+							Value:  "true",
+							Effect: v1.TaintEffectNoSchedule,
+						},
+						{
+							Key:    cloudproviderapi.TaintExternalCloudProvider,
+							Value:  "true",
+							Effect: v1.TaintEffectNoSchedule,
+						},
+					},
+					ProviderID: "node0.cp.12345",
+				},
+				Status: v1.NodeStatus{
+					Conditions: []v1.NodeCondition{
+						{
+							Type:               v1.NodeReady,
+							Status:             v1.ConditionUnknown,
+							LastHeartbeatTime:  metav1.Date(2015, 1, 1, 12, 0, 0, 0, time.UTC),
+							LastTransitionTime: metav1.Date(2015, 1, 1, 12, 0, 0, 0, time.UTC),
+						},
+					},
+					Addresses: []v1.NodeAddress{
+						{
+							Type:    v1.NodeHostName,
+							Address: "node0.cloud.internal",
+						},
+					},
+				},
+			},
+			updatedNode: &v1.Node{
+				ObjectMeta: metav1.ObjectMeta{
+					Name:              "node0",
+					CreationTimestamp: metav1.Date(2012, 1, 1, 0, 0, 0, 0, time.UTC),
+					Annotations: map[string]string{
+						cloudproviderapi.AnnotationAlphaProvidedIPAddr: "10.0.0.1",
+					},
+					Labels: map[string]string{
+						"failure-domain.beta.kubernetes.io/region": "us-west",
+						"failure-domain.beta.kubernetes.io/zone":   "us-west-1a",
+						"topology.kubernetes.io/region":            "us-west",
+						"topology.kubernetes.io/zone":              "us-west-1a",
+						"topology.k8s.cp/zone-id":                  "az1",
+						"my.custom.label/foo":                      "bar",
+					},
+				},
+				Spec: v1.NodeSpec{
+					Taints: []v1.Taint{
+						{
+							Key:    "ImproveCoverageTaint",
+							Value:  "true",
+							Effect: v1.TaintEffectNoSchedule,
+						},
+					},
+					ProviderID: "node0.cp.12345",
+				},
+				Status: v1.NodeStatus{
+					Conditions: []v1.NodeCondition{
+						{
+							Type:               v1.NodeReady,
+							Status:             v1.ConditionUnknown,
+							LastHeartbeatTime:  metav1.Date(2015, 1, 1, 12, 0, 0, 0, time.UTC),
+							LastTransitionTime: metav1.Date(2015, 1, 1, 12, 0, 0, 0, time.UTC),
+						},
+					},
+					Addresses: []v1.NodeAddress{
+						{
+							Type:    v1.NodeInternalIP,
+							Address: "10.0.0.1",
+						},
+						{
+							Type:    v1.NodeExternalIP,
+							Address: "132.143.154.163",
+						},
+						{
+							Type:    v1.NodeHostName,
+							Address: "node0.cloud.internal",
+						},
+					},
+				},
+			},
+		},
+		{
+			name: "[instanceV2] provided additional labels with labels to discard",
+			fakeCloud: &fakecloud.Cloud{
+				EnableInstancesV2: true,
+				Addresses: []v1.NodeAddress{
+					{
+						Type:    v1.NodeInternalIP,
+						Address: "10.0.0.1",
+					},
+					{
+						Type:    v1.NodeExternalIP,
+						Address: "132.143.154.163",
+					},
+				},
+				ExistsByProviderID: true,
+				Err:                nil,
+				Zone: cloudprovider.Zone{
+					FailureDomain: "us-west-1a",
+					Region:        "us-west",
+				},
+				AdditionalLabels: map[string]string{
+					// Kubernetes reserves k8s.io and kubernetes.io namespaces
+					// and should be discarded
+					"topology.kubernetes.io/region": "us-other-west",
+					"topology.k8s.io/region":        "us-other-west",
+					// Should discard labels that already exist
+					"my.custom.label/foo": "bar",
+					"my.custom.label/bar": "foo",
+				},
+			},
+			existingNode: &v1.Node{
+				ObjectMeta: metav1.ObjectMeta{
+					Name:              "node0",
+					CreationTimestamp: metav1.Date(2012, 1, 1, 0, 0, 0, 0, time.UTC),
+					Annotations: map[string]string{
+						cloudproviderapi.AnnotationAlphaProvidedIPAddr: "10.0.0.1",
+					},
+					Labels: map[string]string{
+						"my.custom.label/foo": "fizz",
+						"my.custom.label/bar": "foo",
+					},
+				},
+				Spec: v1.NodeSpec{
+					Taints: []v1.Taint{
+						{
+							Key:    "ImproveCoverageTaint",
+							Value:  "true",
+							Effect: v1.TaintEffectNoSchedule,
+						},
+						{
+							Key:    cloudproviderapi.TaintExternalCloudProvider,
+							Value:  "true",
+							Effect: v1.TaintEffectNoSchedule,
+						},
+					},
+					ProviderID: "node0.cp.12345",
+				},
+				Status: v1.NodeStatus{
+					Conditions: []v1.NodeCondition{
+						{
+							Type:               v1.NodeReady,
+							Status:             v1.ConditionUnknown,
+							LastHeartbeatTime:  metav1.Date(2015, 1, 1, 12, 0, 0, 0, time.UTC),
+							LastTransitionTime: metav1.Date(2015, 1, 1, 12, 0, 0, 0, time.UTC),
+						},
+					},
+					Addresses: []v1.NodeAddress{
+						{
+							Type:    v1.NodeHostName,
+							Address: "node0.cloud.internal",
+						},
+					},
+				},
+			},
+			updatedNode: &v1.Node{
+				ObjectMeta: metav1.ObjectMeta{
+					Name:              "node0",
+					CreationTimestamp: metav1.Date(2012, 1, 1, 0, 0, 0, 0, time.UTC),
+					Annotations: map[string]string{
+						cloudproviderapi.AnnotationAlphaProvidedIPAddr: "10.0.0.1",
+					},
+					Labels: map[string]string{
+						"failure-domain.beta.kubernetes.io/region": "us-west",
+						"failure-domain.beta.kubernetes.io/zone":   "us-west-1a",
+						"topology.kubernetes.io/region":            "us-west",
+						"topology.kubernetes.io/zone":              "us-west-1a",
+						"my.custom.label/foo":                      "fizz",
+						"my.custom.label/bar":                      "foo",
+					},
+				},
+				Spec: v1.NodeSpec{
+					Taints: []v1.Taint{
+						{
+							Key:    "ImproveCoverageTaint",
+							Value:  "true",
+							Effect: v1.TaintEffectNoSchedule,
+						},
+					},
+					ProviderID: "node0.cp.12345",
+				},
+				Status: v1.NodeStatus{
+					Conditions: []v1.NodeCondition{
+						{
+							Type:               v1.NodeReady,
+							Status:             v1.ConditionUnknown,
+							LastHeartbeatTime:  metav1.Date(2015, 1, 1, 12, 0, 0, 0, time.UTC),
+							LastTransitionTime: metav1.Date(2015, 1, 1, 12, 0, 0, 0, time.UTC),
+						},
+					},
+					Addresses: []v1.NodeAddress{
+						{
+							Type:    v1.NodeInternalIP,
+							Address: "10.0.0.1",
+						},
+						{
+							Type:    v1.NodeExternalIP,
+							Address: "132.143.154.163",
+						},
+						{
+							Type:    v1.NodeHostName,
+							Address: "node0.cloud.internal",
+						},
+					},
+				},
+			},
+		},
 		{
 			name: "[instanceV2] provider ID already set",
 			fakeCloud: &fakecloud.Cloud{
@@ -1520,7 +1795,8 @@ func Test_syncNode(t *testing.T) {
 			},
 		},
 		{
-			name: "[instanceV2] provider ID not implemented",
+			name:        "[instanceV2] provider ID not implemented",
+			expectedErr: true,
 			fakeCloud: &fakecloud.Cloud{
 				EnableInstancesV2: true,
 				InstanceTypes:     map[types.NodeName]string{},
@@ -1572,12 +1848,19 @@ func Test_syncNode(t *testing.T) {
 					},
 				},
 				Spec: v1.NodeSpec{
-					Taints: []v1.Taint{},
+					Taints: []v1.Taint{
+						{
+							Key:    cloudproviderapi.TaintExternalCloudProvider,
+							Value:  "true",
+							Effect: v1.TaintEffectNoSchedule,
+						},
+					},
 				},
 			},
 		},
 		{
-			name: "[instanceV2] error getting InstanceMetadata",
+			name:        "[instanceV2] error getting InstanceMetadata",
+			expectedErr: true,
 			fakeCloud: &fakecloud.Cloud{
 				EnableInstancesV2: true,
 				InstanceTypes:     map[types.NodeName]string{},
@@ -1643,10 +1926,14 @@ func Test_syncNode(t *testing.T) {
 
 	for _, test := range tests {
 		t.Run(test.name, func(t *testing.T) {
+			_, ctx := ktesting.NewTestContext(t)
+			ctx, cancel := context.WithCancel(ctx)
+			defer cancel()
+
 			clientset := fake.NewSimpleClientset(test.existingNode)
 			factory := informers.NewSharedInformerFactory(clientset, 0)
 
-			eventBroadcaster := record.NewBroadcaster()
+			eventBroadcaster := record.NewBroadcaster(record.WithContext(ctx))
 			cloudNodeController := &CloudNodeController{
 				kubeClient:                clientset,
 				nodeInformer:              factory.Core().V1().Nodes(),
@@ -1665,9 +1952,12 @@ func Test_syncNode(t *testing.T) {
 			w := eventBroadcaster.StartLogging(klog.Infof)
 			defer w.Stop()
 
-			cloudNodeController.syncNode(context.TODO(), test.existingNode.Name)
+			err := cloudNodeController.syncNode(ctx, test.existingNode.Name)
+			if (err != nil) != test.expectedErr {
+				t.Fatalf("error got: %v expected: %v", err, test.expectedErr)
+			}
 
-			updatedNode, err := clientset.CoreV1().Nodes().Get(context.TODO(), test.existingNode.Name, metav1.GetOptions{})
+			updatedNode, err := clientset.CoreV1().Nodes().Get(ctx, test.existingNode.Name, metav1.GetOptions{})
 			if err != nil {
 				t.Fatalf("error getting updated nodes: %v", err)
 			}
@@ -1679,187 +1969,6 @@ func Test_syncNode(t *testing.T) {
 	}
 }
 
-// test syncNode with instanceV2, same test case with TestGCECondition.
-func TestGCEConditionV2(t *testing.T) {
-	existingNode := &v1.Node{
-		ObjectMeta: metav1.ObjectMeta{
-			Name:              "node0",
-			CreationTimestamp: metav1.Date(2012, 1, 1, 0, 0, 0, 0, time.UTC),
-		},
-		Status: v1.NodeStatus{
-			Conditions: []v1.NodeCondition{
-				{
-					Type:               v1.NodeReady,
-					Status:             v1.ConditionUnknown,
-					LastHeartbeatTime:  metav1.Date(2015, 1, 1, 12, 0, 0, 0, time.UTC),
-					LastTransitionTime: metav1.Date(2015, 1, 1, 12, 0, 0, 0, time.UTC),
-				},
-			},
-		},
-		Spec: v1.NodeSpec{
-			Taints: []v1.Taint{
-				{
-					Key:    cloudproviderapi.TaintExternalCloudProvider,
-					Value:  "true",
-					Effect: v1.TaintEffectNoSchedule,
-				},
-			},
-		},
-	}
-
-	fakeCloud := &fakecloud.Cloud{
-		EnableInstancesV2: true,
-		InstanceTypes: map[types.NodeName]string{
-			types.NodeName("node0"): "t1.micro",
-		},
-		Addresses: []v1.NodeAddress{
-			{
-				Type:    v1.NodeHostName,
-				Address: "node0.cloud.internal",
-			},
-			{
-				Type:    v1.NodeInternalIP,
-				Address: "10.0.0.1",
-			},
-			{
-				Type:    v1.NodeExternalIP,
-				Address: "132.143.154.163",
-			},
-		},
-		Provider: "gce",
-		Err:      nil,
-	}
-
-	clientset := fake.NewSimpleClientset(existingNode)
-	factory := informers.NewSharedInformerFactory(clientset, 0)
-
-	eventBroadcaster := record.NewBroadcaster()
-	cloudNodeController := &CloudNodeController{
-		kubeClient:                clientset,
-		nodeInformer:              factory.Core().V1().Nodes(),
-		nodesLister:               factory.Core().V1().Nodes().Lister(),
-		cloud:                     fakeCloud,
-		recorder:                  eventBroadcaster.NewRecorder(scheme.Scheme, v1.EventSource{Component: "cloud-node-controller"}),
-		nodeStatusUpdateFrequency: 1 * time.Second,
-	}
-
-	stopCh := make(chan struct{})
-	defer close(stopCh)
-
-	factory.Start(stopCh)
-	factory.WaitForCacheSync(stopCh)
-
-	w := eventBroadcaster.StartLogging(klog.Infof)
-	defer w.Stop()
-
-	cloudNodeController.syncNode(context.TODO(), existingNode.Name)
-
-	updatedNode, err := clientset.CoreV1().Nodes().Get(context.TODO(), existingNode.Name, metav1.GetOptions{})
-	if err != nil {
-		t.Fatalf("error getting updated nodes: %v", err)
-	}
-
-	conditionAdded := false
-	for _, cond := range updatedNode.Status.Conditions {
-		if cond.Status == "True" && cond.Type == "NetworkUnavailable" && cond.Reason == "NoRouteCreated" {
-			conditionAdded = true
-		}
-	}
-
-	assert.True(t, conditionAdded, "Network Route Condition for GCE not added by external cloud initializer")
-}
-
-// This test checks that a node with the external cloud provider taint is cloudprovider initialized and
-// the GCE route condition is added if cloudprovider is GCE
-func TestGCECondition(t *testing.T) {
-	existingNode := &v1.Node{
-		ObjectMeta: metav1.ObjectMeta{
-			Name:              "node0",
-			CreationTimestamp: metav1.Date(2012, 1, 1, 0, 0, 0, 0, time.UTC),
-		},
-		Status: v1.NodeStatus{
-			Conditions: []v1.NodeCondition{
-				{
-					Type:               v1.NodeReady,
-					Status:             v1.ConditionUnknown,
-					LastHeartbeatTime:  metav1.Date(2015, 1, 1, 12, 0, 0, 0, time.UTC),
-					LastTransitionTime: metav1.Date(2015, 1, 1, 12, 0, 0, 0, time.UTC),
-				},
-			},
-		},
-		Spec: v1.NodeSpec{
-			Taints: []v1.Taint{
-				{
-					Key:    cloudproviderapi.TaintExternalCloudProvider,
-					Value:  "true",
-					Effect: v1.TaintEffectNoSchedule,
-				},
-			},
-		},
-	}
-
-	fakeCloud := &fakecloud.Cloud{
-		EnableInstancesV2: false,
-		InstanceTypes: map[types.NodeName]string{
-			types.NodeName("node0"): "t1.micro",
-		},
-		Addresses: []v1.NodeAddress{
-			{
-				Type:    v1.NodeHostName,
-				Address: "node0.cloud.internal",
-			},
-			{
-				Type:    v1.NodeInternalIP,
-				Address: "10.0.0.1",
-			},
-			{
-				Type:    v1.NodeExternalIP,
-				Address: "132.143.154.163",
-			},
-		},
-		Provider: "gce",
-		Err:      nil,
-	}
-
-	clientset := fake.NewSimpleClientset(existingNode)
-	factory := informers.NewSharedInformerFactory(clientset, 0)
-
-	eventBroadcaster := record.NewBroadcaster()
-	cloudNodeController := &CloudNodeController{
-		kubeClient:                clientset,
-		nodeInformer:              factory.Core().V1().Nodes(),
-		nodesLister:               factory.Core().V1().Nodes().Lister(),
-		cloud:                     fakeCloud,
-		recorder:                  eventBroadcaster.NewRecorder(scheme.Scheme, v1.EventSource{Component: "cloud-node-controller"}),
-		nodeStatusUpdateFrequency: 1 * time.Second,
-	}
-
-	stopCh := make(chan struct{})
-	defer close(stopCh)
-
-	factory.Start(stopCh)
-	factory.WaitForCacheSync(stopCh)
-
-	w := eventBroadcaster.StartLogging(klog.Infof)
-	defer w.Stop()
-
-	cloudNodeController.syncNode(context.TODO(), existingNode.Name)
-
-	updatedNode, err := clientset.CoreV1().Nodes().Get(context.TODO(), existingNode.Name, metav1.GetOptions{})
-	if err != nil {
-		t.Fatalf("error getting updated nodes: %v", err)
-	}
-
-	conditionAdded := false
-	for _, cond := range updatedNode.Status.Conditions {
-		if cond.Status == "True" && cond.Type == "NetworkUnavailable" && cond.Reason == "NoRouteCreated" {
-			conditionAdded = true
-		}
-	}
-
-	assert.True(t, conditionAdded, "Network Route Condition for GCE not added by external cloud initializer")
-}
-
 func Test_reconcileNodeLabels(t *testing.T) {
 	testcases := []struct {
 		name           string
@@ -1928,6 +2037,11 @@ func Test_reconcileNodeLabels(t *testing.T) {
 
 	for _, test := range testcases {
 		t.Run(test.name, func(t *testing.T) {
+			_, ctx := ktesting.NewTestContext(t)
+			ctx, cancel := context.WithCancel(ctx)
+			defer cancel()
+			stopCh := ctx.Done()
+
 			testNode := &v1.Node{
 				ObjectMeta: metav1.ObjectMeta{
 					Name:   "node01",
@@ -1943,9 +2057,6 @@ func Test_reconcileNodeLabels(t *testing.T) {
 				nodeInformer: factory.Core().V1().Nodes(),
 			}
 
-			stopCh := make(chan struct{})
-			defer close(stopCh)
-
 			// activate node informer
 			factory.Core().V1().Nodes().Informer()
 			factory.Start(stopCh)
@@ -1958,7 +2069,7 @@ func Test_reconcileNodeLabels(t *testing.T) {
 				t.Errorf("unexpected error")
 			}
 
-			actualNode, err := clientset.CoreV1().Nodes().Get(context.TODO(), "node01", metav1.GetOptions{})
+			actualNode, err := clientset.CoreV1().Nodes().Get(ctx, "node01", metav1.GetOptions{})
 			if err != nil {
 				t.Fatalf("error getting updated node: %v", err)
 			}
@@ -2106,6 +2217,10 @@ func TestNodeAddressesChangeDetected(t *testing.T) {
 
 // Test updateNodeAddress with instanceV2, same test case with TestNodeAddressesNotUpdate.
 func TestNodeAddressesNotUpdateV2(t *testing.T) {
+	_, ctx := ktesting.NewTestContext(t)
+	ctx, cancel := context.WithCancel(ctx)
+	defer cancel()
+
 	existingNode := &v1.Node{
 		ObjectMeta: metav1.ObjectMeta{
 			Name:              "node0",
@@ -2163,13 +2278,13 @@ func TestNodeAddressesNotUpdateV2(t *testing.T) {
 		cloud:        fakeCloud,
 	}
 
-	instanceMeta, err := cloudNodeController.getInstanceNodeAddresses(context.TODO(), existingNode)
+	instanceMeta, err := cloudNodeController.getInstanceNodeAddresses(ctx, existingNode)
 	if err != nil {
 		t.Errorf("get instance metadata with error %v", err)
 	}
-	cloudNodeController.updateNodeAddress(context.TODO(), existingNode, instanceMeta)
+	cloudNodeController.updateNodeAddress(ctx, existingNode, instanceMeta)
 
-	updatedNode, err := clientset.CoreV1().Nodes().Get(context.TODO(), existingNode.Name, metav1.GetOptions{})
+	updatedNode, err := clientset.CoreV1().Nodes().Get(ctx, existingNode.Name, metav1.GetOptions{})
 	if err != nil {
 		t.Fatalf("error getting updated nodes: %v", err)
 	}
@@ -2182,6 +2297,10 @@ func TestNodeAddressesNotUpdateV2(t *testing.T) {
 // This test checks that a node with the external cloud provider taint is cloudprovider initialized and
 // and node addresses will not be updated when node isn't present according to the cloudprovider
 func TestNodeAddressesNotUpdate(t *testing.T) {
+	_, ctx := ktesting.NewTestContext(t)
+	ctx, cancel := context.WithCancel(ctx)
+	defer cancel()
+
 	existingNode := &v1.Node{
 		ObjectMeta: metav1.ObjectMeta{
 			Name:              "node0",
@@ -2239,13 +2358,13 @@ func TestNodeAddressesNotUpdate(t *testing.T) {
 		cloud:        fakeCloud,
 	}
 
-	instanceMeta, err := cloudNodeController.getInstanceNodeAddresses(context.TODO(), existingNode)
+	instanceMeta, err := cloudNodeController.getInstanceNodeAddresses(ctx, existingNode)
 	if err != nil {
 		t.Errorf("get instance metadata with error %v", err)
 	}
-	cloudNodeController.updateNodeAddress(context.TODO(), existingNode, instanceMeta)
+	cloudNodeController.updateNodeAddress(ctx, existingNode, instanceMeta)
 
-	updatedNode, err := clientset.CoreV1().Nodes().Get(context.TODO(), existingNode.Name, metav1.GetOptions{})
+	updatedNode, err := clientset.CoreV1().Nodes().Get(ctx, existingNode.Name, metav1.GetOptions{})
 	if err != nil {
 		t.Fatalf("error getting updated nodes: %v", err)
 	}
@@ -2255,15 +2374,16 @@ func TestNodeAddressesNotUpdate(t *testing.T) {
 	}
 }
 
-func TestGetProviderID(t *testing.T) {
+func TestGetInstanceMetadata(t *testing.T) {
 	tests := []struct {
-		name               string
-		fakeCloud          *fakecloud.Cloud
-		existingNode       *v1.Node
-		expectedProviderID string
+		name             string
+		fakeCloud        *fakecloud.Cloud
+		existingNode     *v1.Node
+		expectedMetadata *cloudprovider.InstanceMetadata
+		expectErr        bool
 	}{
 		{
-			name: "node initialized with provider ID",
+			name: "cloud implemented with Instances and provider ID",
 			fakeCloud: &fakecloud.Cloud{
 				EnableInstancesV2: false,
 				InstanceTypes: map[types.NodeName]string{
@@ -2272,6 +2392,9 @@ func TestGetProviderID(t *testing.T) {
 				ExtID: map[types.NodeName]string{
 					types.NodeName("node0"): "12345",
 				},
+				ProviderID: map[types.NodeName]string{
+					types.NodeName("node0"): "fake://12345",
+				},
 				Addresses: []v1.NodeAddress{
 					{
 						Type:    v1.NodeHostName,
@@ -2302,21 +2425,27 @@ func TestGetProviderID(t *testing.T) {
 							Effect: v1.TaintEffectNoSchedule,
 						},
 					},
-					ProviderID: "fake://12345",
 				},
 			},
-			expectedProviderID: "fake://12345",
+			expectedMetadata: &cloudprovider.InstanceMetadata{
+				ProviderID: "fake://12345",
+				NodeAddresses: []v1.NodeAddress{
+					{Type: "Hostname", Address: "node0.cloud.internal"},
+					{Type: "InternalIP", Address: "10.0.0.1"},
+					{Type: "ExternalIP", Address: "132.143.154.163"},
+				},
+			},
 		},
 		{
-			name: "cloud implemented with Instances (without providerID)",
+			name: "cloud implemented with Instances (providerID not implemented)",
 			fakeCloud: &fakecloud.Cloud{
 				EnableInstancesV2: false,
 				InstanceTypes: map[types.NodeName]string{
 					types.NodeName("node0"):        "t1.micro",
 					types.NodeName("fake://12345"): "t1.micro",
 				},
-				ExtID: map[types.NodeName]string{
-					types.NodeName("node0"): "12345",
+				ExtIDErr: map[types.NodeName]error{
+					types.NodeName("node0"): cloudprovider.NotImplemented,
 				},
 				Addresses: []v1.NodeAddress{
 					{
@@ -2340,7 +2469,58 @@ func TestGetProviderID(t *testing.T) {
 					CreationTimestamp: metav1.Date(2012, 1, 1, 0, 0, 0, 0, time.UTC),
 				},
 			},
-			expectedProviderID: "fake://12345",
+			expectedMetadata: &cloudprovider.InstanceMetadata{
+				NodeAddresses: []v1.NodeAddress{
+					{Type: "Hostname", Address: "node0.cloud.internal"},
+					{Type: "InternalIP", Address: "10.0.0.1"},
+					{Type: "ExternalIP", Address: "132.143.154.163"},
+				},
+			},
+		},
+		{
+			name: "cloud implemented with Instances (providerID not implemented) and node with providerID",
+			fakeCloud: &fakecloud.Cloud{
+				EnableInstancesV2: false,
+				InstanceTypes: map[types.NodeName]string{
+					types.NodeName("node0"):        "t1.micro",
+					types.NodeName("fake://12345"): "t1.micro",
+				},
+				ExtIDErr: map[types.NodeName]error{
+					types.NodeName("node0"): cloudprovider.NotImplemented,
+				},
+				Addresses: []v1.NodeAddress{
+					{
+						Type:    v1.NodeHostName,
+						Address: "node0.cloud.internal",
+					},
+					{
+						Type:    v1.NodeInternalIP,
+						Address: "10.0.0.1",
+					},
+					{
+						Type:    v1.NodeExternalIP,
+						Address: "132.143.154.163",
+					},
+				},
+				Err: nil,
+			},
+			existingNode: &v1.Node{
+				ObjectMeta: metav1.ObjectMeta{
+					Name:              "node0",
+					CreationTimestamp: metav1.Date(2012, 1, 1, 0, 0, 0, 0, time.UTC),
+				},
+				Spec: v1.NodeSpec{
+					ProviderID: "fake://asdasd",
+				},
+			},
+			expectedMetadata: &cloudprovider.InstanceMetadata{
+				ProviderID: "fake://asdasd",
+				NodeAddresses: []v1.NodeAddress{
+					{Type: "Hostname", Address: "node0.cloud.internal"},
+					{Type: "InternalIP", Address: "10.0.0.1"},
+					{Type: "ExternalIP", Address: "132.143.154.163"},
+				},
+			},
 		},
 		{
 			name: "cloud implemented with InstancesV2 (with providerID)",
@@ -2353,6 +2533,9 @@ func TestGetProviderID(t *testing.T) {
 				ExtID: map[types.NodeName]string{
 					types.NodeName("node0"): "12345",
 				},
+				ProviderID: map[types.NodeName]string{
+					types.NodeName("node0"): "fake://12345",
+				},
 				Addresses: []v1.NodeAddress{
 					{
 						Type:    v1.NodeHostName,
@@ -2382,13 +2565,20 @@ func TestGetProviderID(t *testing.T) {
 							Effect: v1.TaintEffectNoSchedule,
 						},
 					},
-					ProviderID: "fake://12345",
 				},
 			},
-			expectedProviderID: "fake://12345",
+			expectedMetadata: &cloudprovider.InstanceMetadata{
+				ProviderID: "fake://12345",
+				NodeAddresses: []v1.NodeAddress{
+					{Type: "Hostname", Address: "node0.cloud.internal"},
+					{Type: "InternalIP", Address: "10.0.0.1"},
+					{Type: "ExternalIP", Address: "132.143.154.163"},
+				},
+			},
 		},
-		{
-			name: "cloud implemented with InstancesV2 (without providerID)",
+		{ // it will be requeueud later
+			name:      "cloud implemented with InstancesV2 (without providerID)",
+			expectErr: true,
 			fakeCloud: &fakecloud.Cloud{
 				EnableInstancesV2: true,
 				InstanceTypes: map[types.NodeName]string{
@@ -2429,23 +2619,77 @@ func TestGetProviderID(t *testing.T) {
 					},
 				},
 			},
-			expectedProviderID: "",
+			expectedMetadata: &cloudprovider.InstanceMetadata{
+				NodeAddresses: []v1.NodeAddress{
+					{Type: "Hostname", Address: "node0.cloud.internal"},
+					{Type: "InternalIP", Address: "10.0.0.1"},
+					{Type: "ExternalIP", Address: "132.143.154.163"},
+				},
+			},
+		},
+		{
+			name: "cloud implemented with InstancesV2 (without providerID) and node with providerID",
+			fakeCloud: &fakecloud.Cloud{
+				EnableInstancesV2: true,
+				InstanceTypes: map[types.NodeName]string{
+					types.NodeName("node0"):        "t1.micro",
+					types.NodeName("fake://12345"): "t1.micro",
+				},
+				ExtID: map[types.NodeName]string{
+					types.NodeName("node0"): "12345",
+				},
+				Addresses: []v1.NodeAddress{
+					{
+						Type:    v1.NodeHostName,
+						Address: "node0.cloud.internal",
+					},
+					{
+						Type:    v1.NodeInternalIP,
+						Address: "10.0.0.1",
+					},
+					{
+						Type:    v1.NodeExternalIP,
+						Address: "132.143.154.163",
+					},
+				},
+				Err: nil,
+			},
+			existingNode: &v1.Node{
+				ObjectMeta: metav1.ObjectMeta{
+					Name:              "node0",
+					CreationTimestamp: metav1.Date(2012, 1, 1, 0, 0, 0, 0, time.UTC),
+				},
+				Spec: v1.NodeSpec{
+					ProviderID: "fake://12345",
+				},
+			},
+			expectedMetadata: &cloudprovider.InstanceMetadata{
+				ProviderID:   "fake://12345",
+				InstanceType: "t1.micro",
+				NodeAddresses: []v1.NodeAddress{
+					{Type: "Hostname", Address: "node0.cloud.internal"},
+					{Type: "InternalIP", Address: "10.0.0.1"},
+					{Type: "ExternalIP", Address: "132.143.154.163"},
+				},
+			},
 		},
 	}
 
 	for _, test := range tests {
 		t.Run(test.name, func(t *testing.T) {
+			_, ctx := ktesting.NewTestContext(t)
+
 			cloudNodeController := &CloudNodeController{
 				cloud: test.fakeCloud,
 			}
 
-			providerID, err := cloudNodeController.getProviderID(context.TODO(), test.existingNode)
-			if err != nil {
-				t.Fatalf("error getting provider ID: %v", err)
+			metadata, err := cloudNodeController.getInstanceMetadata(ctx, test.existingNode)
+			if (err != nil) != test.expectErr {
+				t.Fatalf("error expected %v got: %v", test.expectErr, err)
 			}
 
-			if !cmp.Equal(providerID, test.expectedProviderID) {
-				t.Errorf("unexpected providerID %s", cmp.Diff(providerID, test.expectedProviderID))
+			if !cmp.Equal(metadata, test.expectedMetadata) {
+				t.Errorf("unexpected metadata %s", cmp.Diff(metadata, test.expectedMetadata))
 			}
 		})
 	}
@@ -2495,6 +2739,10 @@ func TestUpdateNodeStatus(t *testing.T) {
 	}
 	for _, test := range tests {
 		t.Run(test.name, func(t *testing.T) {
+			_, ctx := ktesting.NewTestContext(t)
+			ctx, cancel := context.WithCancel(ctx)
+			defer cancel()
+
 			fakeCloud := &fakecloud.Cloud{
 				EnableInstancesV2: false,
 				Addresses: []v1.NodeAddress{
@@ -2521,7 +2769,7 @@ func TestUpdateNodeStatus(t *testing.T) {
 			})
 
 			factory := informers.NewSharedInformerFactory(clientset, 0)
-			eventBroadcaster := record.NewBroadcaster()
+			eventBroadcaster := record.NewBroadcaster(record.WithContext(ctx))
 			nodeInformer := factory.Core().V1().Nodes()
 			nodeIndexer := nodeInformer.Informer().GetIndexer()
 			cloudNodeController := &CloudNodeController{
@@ -2542,11 +2790,13 @@ func TestUpdateNodeStatus(t *testing.T) {
 				}
 			}
 
-			w := eventBroadcaster.StartLogging(klog.Infof)
+			w := eventBroadcaster.StartStructuredLogging(0)
 			defer w.Stop()
 
 			start := time.Now()
-			cloudNodeController.UpdateNodeStatus(context.TODO())
+			if err := cloudNodeController.UpdateNodeStatus(ctx); err != nil {
+				t.Fatalf("error updating node status: %v", err)
+			}
 			t.Logf("%d workers: processed %d nodes int %v ", test.workers, test.nodes, time.Since(start))
 			if len(fakeCloud.Calls) != test.nodes {
 				t.Errorf("expected %d cloud-provider calls, got %d", test.nodes, len(fakeCloud.Calls))
