@@ -35,12 +35,6 @@ import (
 	admissionapi "k8s.io/pod-security-admission/api"
 
 	"github.com/onsi/ginkgo/v2"
-	// ensure libs have a chance to initialize
-	_ "github.com/stretchr/testify/assert"
-)
-
-var (
-	pauseImage = imageutils.GetE2EImage(imageutils.Pause)
 )
 
 const (
@@ -73,7 +67,7 @@ func createPodForTaintsTest(hasToleration bool, tolerationSeconds int, podName,
 				Containers: []v1.Container{
 					{
 						Name:  "pause",
-						Image: pauseImage,
+						Image: imageutils.GetE2EImage(imageutils.Pause),
 					},
 				},
 			},
@@ -92,7 +86,7 @@ func createPodForTaintsTest(hasToleration bool, tolerationSeconds int, podName,
 				Containers: []v1.Container{
 					{
 						Name:  "pause",
-						Image: pauseImage,
+						Image: imageutils.GetE2EImage(imageutils.Pause),
 					},
 				},
 				Tolerations: []v1.Toleration{{Key: "kubernetes.io/e2e-evict-taint-key", Value: "evictTaintVal", Effect: v1.TaintEffectNoExecute}},
@@ -111,7 +105,7 @@ func createPodForTaintsTest(hasToleration bool, tolerationSeconds int, podName,
 			Containers: []v1.Container{
 				{
 					Name:  "pause",
-					Image: pauseImage,
+					Image: imageutils.GetE2EImage(imageutils.Pause),
 				},
 			},
 			// default - tolerate forever
@@ -161,7 +155,7 @@ const (
 // - lack of eviction of tolerating pods from a tainted node,
 // - delayed eviction of short-tolerating pod from a tainted node,
 // - lack of eviction of short-tolerating pod after taint removal.
-var _ = SIGDescribe("NoExecuteTaintManager Single Pod [Serial]", func() {
+var _ = SIGDescribe("NoExecuteTaintManager Single Pod", framework.WithSerial(), func() {
 	var cs clientset.Interface
 	var ns string
 	f := framework.NewDefaultFramework("taint-single-pod")
@@ -287,7 +281,7 @@ var _ = SIGDescribe("NoExecuteTaintManager Single Pod [Serial]", func() {
 		Description: The Pod with toleration timeout scheduled on a tainted Node MUST not be
 		evicted if the taint is removed before toleration time ends.
 	*/
-	framework.ConformanceIt("removing taint cancels eviction [Disruptive]", func(ctx context.Context) {
+	framework.ConformanceIt("removing taint cancels eviction", f.WithDisruptive(), func(ctx context.Context) {
 		podName := "taint-eviction-4"
 		pod := createPodForTaintsTest(true, 2*additionalWaitPerDeleteSeconds, podName, podName, ns)
 		observedDeletions := make(chan string, 100)
@@ -370,7 +364,7 @@ var _ = SIGDescribe("NoExecuteTaintManager Single Pod [Serial]", func() {
 	})
 })
 
-var _ = SIGDescribe("NoExecuteTaintManager Multiple Pods [Serial]", func() {
+var _ = SIGDescribe("NoExecuteTaintManager Multiple Pods", framework.WithSerial(), func() {
 	var cs clientset.Interface
 	var ns string
 	f := framework.NewDefaultFramework("taint-multiple-pods")
@@ -447,7 +441,7 @@ var _ = SIGDescribe("NoExecuteTaintManager Multiple Pods [Serial]", func() {
 		Description: In a multi-pods scenario with tolerationSeconds, the pods MUST be evicted as per
 		the toleration time limit.
 	*/
-	framework.ConformanceIt("evicts pods with minTolerationSeconds [Disruptive]", func(ctx context.Context) {
+	framework.ConformanceIt("evicts pods with minTolerationSeconds", f.WithDisruptive(), func(ctx context.Context) {
 		podGroup := "taint-eviction-b"
 		observedDeletions := make(chan string, 100)
 		createTestController(ctx, cs, observedDeletions, podGroup, ns)
